package Email::List::Subscription::Status;

use strict;
use Email::List::Subscription;

sub match
{
    my ($message) = shift(@_);
    my @subscriptions = ();
    my $is_mailman;

    # GNU Mailman
    $is_mailman = defined($message->header('X-MailMan-Version'));
    if (defined($is_mailman))
    {
	push(@subscriptions, _match_mailman($message));
    }

    return @subscriptions;
}

sub _match_mailman
{
    my ($message) = shift(@_);
    my @subscriptions = ();
    my $subject;

    # Check this is a Mailman reminder mail.
    $subject = $message->header('Subject');
    if ($subject !~ m/ mailing list memberships reminder$/)
    {
	# TODO: Error reporting: no listname; no to.
	my $to = $message->header('To');
	my $date = $message->header('Date');

	# Parse the to address. Ignore multiple to addresses.
	my ($to_addr) = Mail::Address->parse($to);

	if (defined($to_addr) && defined($date))
	{
	    # TODO: Preserve case of mailbox.
	    $to = lc($to_addr->address());

	    my @lines = split(/\n/, $message->body());
	    my $has_passwords = 0;
	    my $i;

	    # TODO: Store helpful waffle?
	    for ($i = 0; $i <= $#lines; $i++)
	    {
		if ($lines[$i] =~ m/Passwords for $to:/)
		{
		    $has_passwords = 1;
		    last;
		}
		elsif ($lines[$i] =~ m/Subscriptions for $to:/)
		{
		    last;
		}
	    }

	    for (; $i <= $#lines; $i++)
	    {
		if ($lines[$i] !~ m/^\s*$/)
		{
		    last;
		}
	    }
	
	    # Skip headers for subscription info.
	    for(; $i <= $#lines; $i++)
	    {
		if ($has_passwords)
		{
		    if (   ($lines[$i] !~ m!List\s+Password // URL!)
			&& ($lines[$i] !~ m!-+\s+-+!))
		    {
			last;
		    }
		}
		else
		{
		    if (   ($lines[$i] !~ m!List\s+// URL!)
			&& ($lines[$i] !~ m!-+\s+-+!))
		    {
			last;
		    }
		}
	    }

	    my $listname;
	    my $password;
	    my $url;

	    for (; $i <= $#lines; $i++)
	    {
		if ($has_passwords)
		{
		    ($listname, $password, $url) = split(/\s+/, $lines[$i]);
		}
		else
		{
		    ($listname, $url) = split(/\s+/, $lines[$i]);
		}
	    }

	    if (defined($listname))
	    {
		# TODO: Preserve case of mailbox.
		#
		# We lowercase it here for consistency with other
		# Email::List::Subscription::*.
		$listname = lc($listname);

		my $subscription
		    = new Email::List::Subscription($listname,
						    $to,
						    $Email::List::Subscription::SUBSCRIPTION,
						    $date);

		# Add other optional details.
		if (defined($password))
		{
		    $subscription->add_password($password);
		}

		if (defined($url))
		{
		    $subscription->add_url($url);
		}
	    }
	}
    }

    return @subscriptions;
}

1;
